@@ -198,35 +198,6 @@ void cc_frobenius(point_ptr r, point_ptr p, mpz_ptr q)
     element_pow(r->y, p->y, q);
 }
 
-void curve_init_cc_j(curve_ptr c, element_ptr j)
-//assumes j != 0, 1728
-//TODO: call curve_init_cc_ab instead of duplicating code
-{
-    common_curve_ptr cc;
-
-    c->field = j->field;
-    c->random = cc_random;
-    c->from_x = cc_from_x;
-    c->from_hash = cc_from_hash;
-    c->neg = cc_neg;
-    c->add = cc_add;
-    c->mul = cc_mul;
-    c->data = malloc(sizeof(common_curve_t));
-    c->curve_clear = cc_clear;
-    cc = c->data;
-    element_init(cc->a, c->field);
-    element_init(cc->b, c->field);
-    element_set_si(cc->a, 1728);
-    element_sub(cc->a, cc->a, j);
-    element_invert(cc->a, cc->a);
-    element_mul(cc->a, cc->a, j);
-
-    //b = 2 j / (1728 - j)
-    element_add(cc->b, cc->a, cc->a);
-    //a = 3 j / (1728 - j)
-    element_add(cc->a, cc->a, cc->b);
-}
-
 void curve_init_cc_ab(curve_ptr c, element_ptr a, element_ptr b)
 {
     common_curve_ptr cc;
@@ -246,12 +217,34 @@ void curve_init_cc_ab(curve_ptr c, element_ptr a, element_ptr b)
     element_set(cc->b, b);
 }
 
+void curve_init_cc_j(curve_ptr c, element_ptr j)
+//assumes j != 0, 1728
+{
+    element_t a, b;
+    element_init(a, j->field);
+    element_init(b, j->field);
+
+    element_set_si(a, 1728);
+    element_sub(a, a, j);
+    element_invert(a, a);
+    element_mul(a, a, j);
+
+    //b = 2 j / (1728 - j)
+    element_add(b, a, a);
+    //a = 3 j / (1728 - j)
+    element_add(a, a, b);
+    curve_init_cc_ab(c, a, b);
+
+    element_clear(a);
+    element_clear(b);
+}
+
 void twist_curve(curve_ptr c)
     //WARNING: existing points will no longer lie on c
     //as this modifies c in place
 {
     common_curve_ptr cc = c->data;
-    element_ptr nqr = c->field->nqr;
+    element_ptr nqr = field_get_nqr(c->field);
     element_mul(cc->a, cc->a, nqr);
     element_mul(cc->a, cc->a, nqr);
     element_mul(cc->b, cc->b, nqr);
@@ -259,12 +252,13 @@ void twist_curve(curve_ptr c)
     element_mul(cc->b, cc->b, nqr);
 }
 
-//TODO: curve_clear_cc_ab
+//TODO: curve_clear
 
-void cc_switch_field(curve_ptr cnew, curve_ptr c, fieldmap_ptr map)
+void cc_switch_field(curve_ptr cnew, curve_ptr c,
+	field_ptr dstfield, fieldmap map)
 {
     common_curve_ptr ccnew, cc;
-    cnew->field = map->dstfield;
+    cnew->field = dstfield;
     cnew->random = c->random;
     cnew->neg = c->neg;
     cnew->add = c->add;
@@ -275,14 +269,8 @@ void cc_switch_field(curve_ptr cnew, curve_ptr c, fieldmap_ptr map)
     cc = c->data;
     element_init(ccnew->a, cnew->field);
     element_init(ccnew->b, cnew->field);
-    map->map(ccnew->a, cc->a);
-    map->map(ccnew->b, cc->b);
-}
-
-void cc_init_extend(curve_ptr cnew, curve_ptr c, field_ptr fext)
-{
-    //assumes fext is a field extension of the field c is defined over
-    cc_switch_field(cnew, c, ((polymod_field_data_ptr) fext->data)->mapbase);
+    map(ccnew->a, cc->a);
+    map(ccnew->b, cc->b);
 }
 
 void compute_trace_n(mpz_t res, mpz_t q, mpz_t trace, int n)
@@ -313,189 +301,17 @@ void compute_trace_n(mpz_t res, mpz_t q, mpz_t trace, int n)
     mpz_clear(c0);
 }
 
-void point_map(point_t R, fieldmap_t map, point_t P)
+void point_map(point_t R, fieldmap map, point_t P)
 {
     if (point_is_inf(P)) {
 	point_set_inf(R);
 	return;
     }
-    map->map(R->x, P->x);
-    map->map(R->y, P->y);
+    map(R->x, P->x);
+    map(R->y, P->y);
     R->inf_flag = 0;
 }
 
-void point_extend(point_t R, point_t P, field_t f)
-{
-    point_map(R, ((polymod_field_data_ptr) f->data)->mapbase, P);
-}
-
-//These slow_miller_*() functions are more general
-//e.g. can use to compute Weil pairing not just Tate pairing.
-//TODO: test these
-void slow_miller_vertical(element_t v, element_t vd, point_t Q, point_t Z)
-{
-    element_t t0;
-
-    if (point_is_inf(Z)) {
-	return;
-    }
-    element_init(t0, v->field);
-    element_sub(t0, Q->x, Z->x);
-    element_mul(v, v, t0);
-    element_clear(t0);
-
-    return;
-}
-
-void slow_miller_tangent(element_t v, element_t vd, point_t Q, point_t Z)
-{
-    //a = -slope_tangent(Z.x, Z.y);
-    //b = 1;
-    //c = -(Z.y + a * Z.x);
-    //but we multiply by 2*Z.y to avoid division
-
-    //a = -Zx * (Zx + Zx + Zx + *twicea2) - *a4;
-    //Common curves: a2 = 0
-    element_t a, b, c;
-    element_t t0;
-    common_curve_ptr cc = Z->curve->data;
-
-    if (point_is_inf(Z)) {
-	return;
-    }
-
-    if (element_is0(Z->y)) {
-	//order 2
-	slow_miller_vertical(v, vd, Q, Z);
-	return;
-    }
-
-    element_init(a, v->field);
-    element_init(b, v->field);
-    element_init(c, v->field);
-    element_init(t0, v->field);
-
-    element_add(a, Z->x, Z->x);
-    element_add(a, a, Z->x);
-    element_mul(a, a, Z->x);
-    element_add(a, a, cc->a);
-    element_neg(a, a);
-
-    /* alternative
-    element_set_si(a, 3);
-    element_mul(a, a, Z->x);
-    element_mul(a, a, Z->x);
-    element_add(a, a, cc->a);
-    element_neg(a, a);
-    */
-
-    element_add(b, Z->y, Z->y);
-
-    element_mul(t0, b, Z->y);
-    element_mul(c, a, Z->x);
-    element_add(c, c, t0);
-    element_neg(c, c);
-
-    element_mul(vd, vd, b);
-    element_mul(a, a, Q->x);
-    element_mul(b, b, Q->y);
-    element_add(c, c, a);
-    element_add(c, c, b);
-    element_mul(v, v, c);
-
-    element_clear(a);
-    element_clear(b);
-    element_clear(c);
-    element_clear(t0);
-}
-
-void slow_miller_line(element_t v, element_t vd, point_t Q, point_t A, point_t B)
-{
-    //a = -(Q.y - P.y) / (Q.x - P.x);
-    //b = 1;
-    //c = -(P.y + a * P.x);
-    //but we'll multiply by Q.x - P.x to avoid division
-    element_t a, b, c;
-    element_t t0;
-
-    //we assume B is never O
-    if (point_is_inf(A)) {
-	slow_miller_vertical(v, vd, Q, A);
-	return;
-    }
-
-    if (!element_cmp(A->x, B->x)) {
-	if (!element_cmp(A->y, B->y)) {
-	    slow_miller_tangent(v, vd, Q, A);
-	    return;
-	}
-	slow_miller_vertical(v, vd, Q, A);
-	return;
-    }
-
-    element_init(a, v->field);
-    element_init(b, v->field);
-    element_init(c, v->field);
-    element_init(t0, v->field);
-
-    element_sub(b, B->x, A->x);
-    element_sub(a, A->y, B->y);
-    element_mul(t0, b, A->y);
-    element_mul(c, a, A->x);
-    element_add(c, c, t0);
-    element_neg(c, c);
-
-    element_mul(vd, vd, b);
-    element_mul(a, a, Q->x);
-    element_mul(b, b, Q->y);
-    element_add(c, c, a);
-    element_add(c, c, b);
-    element_mul(v, v, c);
-
-    element_clear(a);
-    element_clear(b);
-    element_clear(c);
-    element_clear(t0);
-}
-
-void slow_miller(element_t res, mpz_t q, point_t P, point_t Q)
-{
-    //collate divisions
-    int m;
-    element_t v, vd;
-    point_t Z;
-    element_init(v, res->field);
-    element_init(vd, res->field);
-    point_init(Z, P->curve);
-
-    point_set(Z, P);
-
-    element_set1(v);
-    element_set1(vd);
-    m = mpz_sizeinbase(q, 2) - 2;
-
-    while(m >= 0) {
-	element_mul(v, v, v);
-	element_mul(vd, vd, vd);
-	slow_miller_tangent(v, vd, Q, Z);
-	point_add(Z, Z, Z);
-	slow_miller_vertical(vd, v, Q, Z);
-	if (mpz_tstbit(q, m)) {
-	    slow_miller_line(v, vd, Q, Z, P);
-	    point_add(Z, Z, P);
-	    slow_miller_vertical(vd, v, Q, Z);
-	}
-	m--;
-    }
-
-    element_invert(vd, vd);
-    element_mul(res, v, vd);
-
-    element_clear(v);
-    element_clear(vd);
-    point_clear(Z);
-}
-
 void cc_miller_vertical(element_t v, point_t Q, point_t Z)
 {
     element_t t0;
@@ -614,7 +430,8 @@ void cc_miller_line(element_t v, point_t Q, point_t A, point_t B)
 
 //assumes P is in the base field, Q in some field extension,
 //and will be used in Tate pairing computation
-void cc_miller(element_t res, mpz_t q, point_t Pbase, point_t Q)
+void cc_miller(element_t res, mpz_t q, point_t Pbase, point_t Q,
+	fieldmap mapbase)
 //TODO: optimize using the fact that P lies in the base field
 //(we don't need Q until the last minute in miller_*()'s)
 {
@@ -629,7 +446,7 @@ void cc_miller(element_t res, mpz_t q, point_t Pbase, point_t Q)
     point_init(Z, Q->curve);
     point_init(P, Q->curve);
 
-    point_extend(P, Pbase, (Q->curve->field));
+    point_map(P, mapbase, Pbase);
     point_set(Z, P);
 
     element_set1(v);
