@@ -18,7 +18,6 @@ static void deterministic_mpz_random(mpz_t z, mpz_t limit, void *data)
 
 static void file_mpz_random(mpz_t r, mpz_t limit, void *data)
 //TODO: generate some kind of warning on error
-//also allow programmer to pick between /dev/random and /dev/urandom
 {
     char *filename = (char *) data;
     FILE *fp;
@@ -26,7 +25,6 @@ static void file_mpz_random(mpz_t r, mpz_t limit, void *data)
     unsigned char *bytes;
     mpz_t z;
     mpz_init(z);
-    //fp = fopen("/dev/urandom", "rb");
     fp = fopen(filename, "rb");
     if (!fp) return;
     n = mpz_sizeinbase(limit, 2);
@@ -181,14 +179,16 @@ static void fp_tonelli(element_ptr x, element_ptr a)
     mpz_t e;
     mpz_t t, t0;
     element_t ginv, e0;
+    element_ptr nqr;
 
     mpz_init(t);
     mpz_init(e);
     mpz_init(t0);
     element_init(ginv, a->field);
     element_init(e0, a->field);
+    nqr = field_get_nqr(a->field);
 
-    element_invert(ginv, a->field->nqr); 
+    element_invert(ginv, nqr); 
 
     //let q be the order of the field
     //q - 1 = 2^s t, t odd
@@ -210,7 +210,7 @@ static void fp_tonelli(element_ptr x, element_ptr a)
     mpz_tdiv_q_2exp(t, t, 1);
     element_pow(e0, e0, t);
     mpz_tdiv_q_2exp(e, e, 1);
-    element_pow(x, x->field->nqr, e);
+    element_pow(x, nqr, e);
     element_mul(x, x, e0);
     mpz_clear(t);
     mpz_clear(e);
@@ -222,8 +222,6 @@ static void fp_tonelli(element_ptr x, element_ptr a)
 static void zp_field_clear(field_t f)
 {
     mpz_clear(f->order);
-    mpz_clear(f->nqr->data);
-    free(f->nqr->data);
 }
 
 static int zp_to_bytes(unsigned char *data, element_t e)
@@ -269,6 +267,7 @@ static int zp_from_bytes(element_t e, unsigned char *data)
 
 void field_init_fp(field_ptr f, mpz_t prime)
 {
+    field_init(f);
     f->init = zp_init;
     f->clear = zp_clear;
     f->set_si = zp_set_si;
@@ -296,12 +295,6 @@ void field_init_fp(field_ptr f, mpz_t prime)
     mpz_init(f->order);
     mpz_set(f->order, prime);
     f->data = NULL;
-    f->nqr->data = malloc(sizeof(mpz_t));
-    mpz_init(f->nqr->data);
-    for(;;) {
-	pbc_mpz_random(f->nqr->data, prime);
-	if (mpz_legendre(f->nqr->data, prime) == -1) break;
-    }
     f->fixed_length_in_bytes = (mpz_sizeinbase(prime, 2) + 7) / 8;
 }
 
@@ -322,3 +315,15 @@ void element_pow(element_ptr x, element_ptr a, mpz_ptr n)
     element_set(x, result);
     element_clear(result);
 }
+
+element_ptr field_get_nqr(field_ptr f)
+{
+    if (!f->nqr) {
+	f->nqr = malloc(sizeof(element_t));
+	element_init(f->nqr, f);
+	do {
+	    element_random(f->nqr);
+	} while (element_is_sqr(f->nqr));
+    }
+    return f->nqr;
+}
