@@ -1,4 +1,5 @@
 #include "pairing.h"
+#include "fieldi.h"
 
 static void curve_group_init(element_ptr e)
 {
@@ -116,7 +117,7 @@ static void cc_pairing(element_ptr out, element_ptr in1, element_ptr in2,
 	pairing_t pairing)
 {
     mnt_pairing_data_ptr p = pairing->data;
-    cc_miller(out, pairing->r, in1->data, in2->data);
+    cc_miller(out, pairing->r, in1->data, in2->data, p->mapbase);
     element_pow(out, out, p->tateexp);
 }
 
@@ -131,8 +132,8 @@ static int cc_is_almost_coddh(element_ptr a, element_ptr b,
     element_init(t1, pairing->GT);
     element_init(t2, pairing->GT);
     mnt_pairing_data_ptr p = pairing->data;
-    cc_miller(t0, pairing->r, a->data, d->data);
-    cc_miller(t1, pairing->r, b->data, c->data);
+    cc_miller(t0, pairing->r, a->data, d->data, p->mapbase);
+    cc_miller(t1, pairing->r, b->data, c->data, p->mapbase);
     element_pow(t0, t0, p->tateexp);
     element_pow(t1, t1, p->tateexp);
     element_mul(t2, t0, t1);
@@ -206,7 +207,13 @@ void pairing_init_cc_param(pairing_t pairing, cc_param_t param)
     mpz_sub_ui(p->tateexp, p->Fqk->order, 1);
     mpz_divexact(p->tateexp, p->tateexp, pairing->r);
 
-    cc_init_extend(p->Eqk, p->Eq, p->Fqk);
+    p->mapbase = ((polymod_field_data_ptr) p->Fqk->data)->mapbase;
+
+    cc_switch_field(p->Eqk, p->Eq, p->Fqk, p->mapbase);
+
+    pairing->G1 = malloc(sizeof(field_t));
+    pairing->G2 = malloc(sizeof(field_t));
+
     field_init_curve_group(pairing->G1, p->Eq, param->h);
     mpz_init(z);
     mpz_set_si(z, 1);
@@ -215,4 +222,75 @@ void pairing_init_cc_param(pairing_t pairing, cc_param_t param)
     p->k = param->k;
     pairing->GT = p->Fqk;
     pairing->phi = trace;
+
+    element_clear(a);
+    element_clear(b);
+}
+
+static void phi_identity(element_ptr out, element_ptr in, pairing_ptr pairing)
+{
+    element_set(out, in);
+}
+
+//TODO: use Miller algorithm that's been specialized for Solinas primes
+static void solinas_pairing(element_ptr out, element_ptr in1, element_ptr in2,
+	pairing_t pairing)
+//in1, in2 are from E(F_q), out from F_q^2
+{
+    solinas_pairing_data_ptr p = pairing->data;
+    point_t Q;
+    point_ptr P = in2->data;
+    fi_data_ptr fdp;
+
+    point_init(Q, p->Eq2);
+    Q->inf_flag = 0;
+    fdp = Q->x->data;
+    element_neg(fdp->x, P->x);
+    element_set0(fdp->y);
+    fdp = Q->y->data;
+    element_set(fdp->y, P->y);
+    element_set0(fdp->x);
+
+    cc_miller(out, pairing->r, in1->data, Q, element_field_to_fieldi);
+    element_pow(out, out, p->tateexp);
+    point_clear(Q);
+}
+
+void pairing_init_solinas_param(pairing_t pairing, solinas_param_t param)
+{
+    element_t a, b;
+    solinas_pairing_data_ptr p;
+
+    p =	pairing->data = malloc(sizeof(solinas_pairing_data_t));
+    p->exp2 = param->exp2;
+    p->exp1 = param->exp1;
+    p->sign1 = param->sign1;
+    p->sign0 = param->sign0;
+    mpz_init(pairing->r);
+    mpz_set(pairing->r, param->r);
+    field_init_fp(pairing->Zr, pairing->r);
+    pairing->map = solinas_pairing;
+    pairing->is_almost_coddh = cc_is_almost_coddh;
+
+    field_init_fp(p->Fq, param->q);
+    element_init(a, p->Fq);
+    element_init(b, p->Fq);
+    element_set1(a);
+    element_set0(b);
+    curve_init_cc_ab(p->Eq, a, b);
+    element_clear(a);
+    element_clear(b);
+
+    field_init_fi(p->Fq2, p->Fq);
+    cc_switch_field(p->Eq2, p->Eq, p->Fq2, element_field_to_fieldi);
+
+    mpz_init(p->tateexp);
+    mpz_sub_ui(p->tateexp, p->Fq2->order, 1);
+    mpz_divexact(p->tateexp, p->tateexp, pairing->r);
+
+    pairing->G1 = malloc(sizeof(field_t));
+    field_init_curve_group(pairing->G1, p->Eq, param->h);
+    pairing->G2 = pairing->G1;
+    pairing->phi = phi_identity;
+    pairing->GT = p->Fq2;
 }
