@@ -465,12 +465,11 @@ void field_init_poly(field_ptr f, field_ptr base_field)
 {
     poly_field_data_ptr p;
 
+    field_init(f);
     f->data = malloc(sizeof(poly_field_data_t));
     p = f->data;
     p->field = base_field;
-    p->mapbase->srcfield = base_field;
-    p->mapbase->dstfield = f;
-    p->mapbase->map = poly_basemap;
+    p->mapbase = poly_basemap;
     f->field_clear = field_clear_poly;
     f->init = poly_init;
     f->clear = poly_clear;
@@ -505,7 +504,6 @@ static void field_clear_polymod(field_ptr f)
 
 static int polymod_is_sqr(element_ptr e)
 {
-    //compute a nqr
     int res;
     mpz_t z;
     element_t e0;
@@ -559,7 +557,7 @@ static void polymod_sqrt(element_ptr res, element_ptr a)
 	}
 	element_set1(s);
 	//TODO: this can be optimized greatly
-	//since we know r has to form ax + b
+	//since we know r has the form ax + b
 	for (i = mpz_sizeinbase(z, 2) - 1; i >=0; i--) {
 	    element_mul(s, s, s);
 	    if (poly_degree(s) == 2) {
@@ -697,7 +695,7 @@ void compute_x_powers(darray_ptr xpwr, element_ptr poly)
     element_clear(p0);
 }
 
-void field_init_polymod_no_nqr(field_ptr f, element_ptr poly)
+void field_init_polymod(field_ptr f, element_ptr poly)
     //assumes poly is monic
 {
     poly_field_data_ptr pdp = poly->field->data;
@@ -705,12 +703,11 @@ void field_init_polymod_no_nqr(field_ptr f, element_ptr poly)
     polymod_field_data_ptr p;
     int n;
 
+    field_init(f);
     f->data = malloc(sizeof(polymod_field_data_t));
     p = f->data;
     p->field = pdp->field;
-    p->mapbase->srcfield = p->field;
-    p->mapbase->dstfield = f;
-    p->mapbase->map = poly_basemap;
+    p->mapbase = poly_basemap;
     element_init(p->poly, poly->field);
     element_set(p->poly, poly);
     f->field_clear = field_clear_polymod;
@@ -750,25 +747,6 @@ void field_init_polymod_no_nqr(field_ptr f, element_ptr poly)
     compute_x_powers(p->xpwr, poly);
 }
 
-void field_init_polymod(field_ptr f, element_ptr poly)
-{
-    element_t e0;
-    mpz_t z;
-    field_init_polymod_no_nqr(f, poly);
-    //compute a nqr
-    element_init(f->nqr, f);
-    element_init(e0, f);
-    mpz_init(z);
-    mpz_sub_ui(z, f->order, 1);
-    mpz_divexact_ui(z, z, 2);
-
-    do {
-	element_random(f->nqr);
-	element_pow(e0, f->nqr, z);
-    } while (element_is1(e0));
-    element_clear(e0);
-}
-
 void trial_divide(darray_ptr factor, darray_ptr mult, mpz_t n)
 {
     mpz_t p, m;
@@ -850,7 +828,7 @@ int poly_is_irred_degfac(element_ptr f, darray_t factor)
     mpz_init(deg);
     mpz_init(z);
 
-    field_init_polymod_no_nqr(rxmod, f);
+    field_init_polymod(rxmod, f);
 
     mpz_set_ui(deg, poly_degree(f));
     element_init(xpow, rxmod);
